//
//  AddCardModel.m
//  pay
//
//  Created by Jasmin Suljic on 21/12/16.
//  Copyright © 2019 Monri Payments d.o.o. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "AddCardModel.h"
#import "PayModel.h"
#import "InputResolver.h"
#import "Customer.h"

@implementation AddCardModel

- (id)init:(NSString *)merchantId client:(NSString *)client orderInfo:(NSString *)orderInfo orderNumber:(NSString *)orderNumber sign:(NSString *)sign amount:(NSString *)amount currency:(NSString *)currency clientSecret:(NSString *)clientSecret authToken:(NSString *)authToken {
    return [self init:merchantId client:client orderInfo:orderInfo orderNumber:orderNumber sign:sign amount:amount currency:currency clientSecret:clientSecret authToken:authToken customer:nil params :nil];
}

- (id)init:(NSString *)merchantId client:(NSString *)client orderInfo:(NSString *)orderInfo orderNumber:(NSString *)orderNumber sign:(NSString *)sign amount:(NSString *)amount currency:(NSString *)currency clientSecret:(NSString *)clientSecret authToken:(NSString *)authToken customer:(Customer *)customer {
    return [self init:merchantId client:client orderInfo:orderInfo orderNumber:orderNumber sign:sign amount:amount currency:currency clientSecret:clientSecret authToken:authToken customer:customer params :nil];
}

- (id)init:(NSString *)merchantId client:(NSString *)client orderInfo:(NSString *)orderInfo orderNumber:(NSString *)orderNumber sign:(NSString *)sign amount:(NSString *)amount currency:(NSString *)currency clientSecret:(NSString *)clientSecret authToken:(NSString *)authToken customer:(Customer *)customer params:(NSDictionary *)params {
    _merchantId = merchantId;
    _client = client;
    _orderInfo = orderInfo;
    _orderNumber = orderNumber;
    _sign = sign;
    _amount = amount;
    _currency = currency;
    _clientSecret = clientSecret;
    _authToken = authToken;
    _digest = [PayModel sha1:[NSString stringWithFormat:@"%@%@%@%@%@%@%@",
                                                        _merchantId,
                                                        _authToken,
                                                        _orderNumber,
                                                        _sign,
                                                        _amount,
                                                        _currency,
                                                        _clientSecret
    ]];
    _customer = customer;
    _params = params;

    return [self init];
}

- (id)init:(Merchant *)merchant orderDetails:(OrderDetails *)orderDetails customer:(Customer *)customer {
    _merchantId = [merchant merchantId];
    _client = [orderDetails client];
    _orderInfo = [orderDetails orderInfo];
    _orderNumber = [orderDetails orderNumber];
    _sign = [orderDetails signature];
    _amount = [orderDetails amount];
    _currency = [orderDetails currency];
    _clientSecret = [merchant clientSecret];
    _authToken = [merchant authenticationToken];
    _digest = [PayModel sha1:[NSString stringWithFormat:@"%@%@%@%@%@%@%@",
                              _merchantId,
                              _authToken,
                              _orderNumber,
                              _sign,
                              _amount,
                              _currency,
                              _clientSecret
                              ]];
    _customer = customer;
    
    return [self init];
}


- (NSString *)toInput {
    NSMutableDictionary *data = [[NSMutableDictionary alloc] init];

    [data setValue:_client forKey:@"client"];
    [data setValue:_merchantId forKey:@"merchant_id"];
    [data setValue:_orderInfo forKey:@"order_info"];
    [data setValue:_orderNumber forKey:@"order_number"];
    [data setValue:_sign forKey:@"sign"];
    [data setValue:_amount forKey:@"amount"];
    [data setValue:_currency forKey:@"currency"];
    [data setValue:_clientSecret forKey:@"client_secret"];
    [data setValue:_authToken forKey:@"auth_token"];
    [data setValue:_digest forKey:@"digest"];

    if (_customer) {
        [data addEntriesFromDictionary:[_customer toDictionary]];
    }

    if (_params) {
        [data setValue:_params forKey:@"params"];
    } else {
        [data setValue:[[NSDictionary alloc] init] forKey:@"params"];
    }

    return [InputResolver encode:data];
}


@end
