//
//  RedirectResolver.m
//  pay
//
//  Created by Jasmin Suljic on 21/12/16.
//  Copyright © 2016 Webteh.hr. All rights reserved.
//

#import "RedirectResolver.h"
#import "SharedLifecycleDelegate.h"
#import "StartedTransactionResolverRunnable.h"
#import "TransactionFailureResolverRunnable.h"
#import "SuccessTransactionResolverRunnable.h"
#import "TimeoutResolverRunnable.h"
#import "AddPaymentMethodSuccessRunnable.h"
#import "ThreeDSResolverRunnable.h"
#import "RedirectResolverRunnable.h"

const int STARTED_TRANSACTION = 0;
const int TRANSACTION_SUCCESS = 1;
const int TRANSACTION_FAILURE = 2;
const int TIMEOUT = 3;
const int UNKNOWN = 4;
const int THREE_DS = 5;
const int STARTED_TRANSACTION_3DS = 6;
const int IGNORE = 2121;
const int ADD_PAYMENT_METHOD_STARTED_TRANSACTION = 101;
const int ADD_PAYMENT_METHOD_THREE_DS = 102;
const int ADD_PAYMENT_METHOD_STARTED_TRANSACTION_3DS = 103;
const int ADD_PAYMENT_METHOD_SUCCESS = 104;
const int ADD_PAYMENT_METHOD_FAILURE = 105;


@interface RedirectResolver()
@property long timePassed;
@property long timeout;
@property long timeoutHandlerPeriod;
@property(strong, nonatomic) NSTimer* timer;
@property(strong, nonatomic) NSDictionary* resolvedUrlResultMap;
@property(strong, nonatomic) NSDictionary* resolvedUrlRunnableMap;

@end

@implementation RedirectResolver

-(id) init: (TransactionLifecycleAdapter *) adapter
   timeout: (long) timeout
{
    self.transactionInProgress = false;
    _timePassed = 0;
    _timeout = timeout;
    _timeoutHandlerPeriod = 500;
    _delegate = [adapter delegate];
    [self addResolveUrlItems];
    [self addResolvedUrlRunnable:adapter];
    return [self init];
}

-(id) initAddPayment:(AddPaymentMethodLifecycleAdapter *)adapter timeout:(long)timeout {
    self.transactionInProgress = false;
    _timePassed = 0;
    _timeout = timeout;
    _timeoutHandlerPeriod = 500;
    _delegate = [adapter delegate];
    [self addResolveUrlItems];
    [self addResolvedUrlRunnableAddPaymentMethod:adapter];
    return [self init];
}

-(void) addResolveUrlItems {
    _resolvedUrlResultMap = [NSMutableDictionary dictionary];
    
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:IGNORE] forKey:@"/default/add-card"];
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:IGNORE] forKey:@"/default/pay-cvv"];
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:IGNORE] forKey:@"/default/add-payment-method"];
    
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:STARTED_TRANSACTION] forKey:@"/api/add-card"];
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:STARTED_TRANSACTION] forKey:@"/api/pay-cvv"];
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:STARTED_TRANSACTION_3DS] forKey:@"/api/pares"];
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:THREE_DS] forKey:@"/default/3ds"];
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:TRANSACTION_SUCCESS] forKey:@"/api/success"];
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:TRANSACTION_FAILURE] forKey:@"/api/error"];
    
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:ADD_PAYMENT_METHOD_STARTED_TRANSACTION] forKey:@"/api/process-add-payment-method"];
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:ADD_PAYMENT_METHOD_STARTED_TRANSACTION_3DS] forKey:@"/api/add-payment-method-pares"];
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:ADD_PAYMENT_METHOD_THREE_DS] forKey:@"/default/add-payment-method-3ds"];
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:ADD_PAYMENT_METHOD_SUCCESS] forKey:@"/api/add-payment-method-success"];
    [_resolvedUrlResultMap setValue:[NSNumber numberWithInt:ADD_PAYMENT_METHOD_FAILURE] forKey:@"/api/add-payment-method-error"];
}

-(void) addResolvedUrlRunnable: (TransactionLifecycleAdapter *) adapter {
    _resolvedUrlRunnableMap = [NSMutableDictionary dictionary];
    
    id<SharedLifecycleDelegate> delegate = (id<SharedLifecycleDelegate>) [adapter delegate];
    
    StartedTransactionResolverRunnable* startedTransactionResolverRunnable = [[StartedTransactionResolverRunnable alloc] init: delegate];
    SuccessTransactionResolverRunnable* successTransactionResolverRunnable = [[SuccessTransactionResolverRunnable alloc] init: adapter.delegate];
    ThreeDSResolverRunnable* threeDSResolverRunnable = [[ThreeDSResolverRunnable alloc] init:delegate];
    TimeoutResolverRunnable* timeoutResolverRunnable = [[TimeoutResolverRunnable alloc] init:delegate onTimeout:^{
        [adapter timeout];
    }];
    
    TransactionFailureResolverRunnable* transactionFailureResolverRunnable = [[TransactionFailureResolverRunnable alloc] init: delegate];
    
    [_resolvedUrlRunnableMap setValue:startedTransactionResolverRunnable forKey:[[NSNumber numberWithInt:STARTED_TRANSACTION] stringValue]];
    [_resolvedUrlRunnableMap setValue:startedTransactionResolverRunnable forKey:[[NSNumber numberWithInt:STARTED_TRANSACTION_3DS] stringValue]];
    
    [_resolvedUrlRunnableMap setValue:successTransactionResolverRunnable forKey:[[NSNumber numberWithInt:TRANSACTION_SUCCESS] stringValue]];
    
    [_resolvedUrlRunnableMap setValue:threeDSResolverRunnable forKey:[[NSNumber numberWithInt:THREE_DS] stringValue]];
    
    [_resolvedUrlRunnableMap setValue:timeoutResolverRunnable forKey:[[NSNumber numberWithInt:TIMEOUT] stringValue]];
    
    [_resolvedUrlRunnableMap setValue:transactionFailureResolverRunnable forKey:[[NSNumber numberWithInt:TRANSACTION_FAILURE] stringValue]];
}

-(void) addResolvedUrlRunnableAddPaymentMethod: (AddPaymentMethodLifecycleAdapter *) adapter {
    _resolvedUrlRunnableMap = [NSMutableDictionary dictionary];
    id<SharedLifecycleDelegate> delegate = (id<SharedLifecycleDelegate>) [adapter delegate];
    
    StartedTransactionResolverRunnable* startedTransactionResolverRunnable = [[StartedTransactionResolverRunnable alloc] init: delegate];
    ThreeDSResolverRunnable* threeDSResolverRunnable = [[ThreeDSResolverRunnable alloc] init:delegate];
    TimeoutResolverRunnable* timeoutResolverRunnable = [[TimeoutResolverRunnable alloc] init:delegate onTimeout:^{
        [adapter timeout];
    }];
    TransactionFailureResolverRunnable* transactionFailureResolverRunnable = [[TransactionFailureResolverRunnable alloc] init: delegate];
    
    AddPaymentMethodSuccessRunnable* addPaymentMethodSuccessRunnable = [[AddPaymentMethodSuccessRunnable alloc] init: adapter.delegate];
    
    [_resolvedUrlRunnableMap setValue:startedTransactionResolverRunnable forKey:[[NSNumber numberWithInt:ADD_PAYMENT_METHOD_STARTED_TRANSACTION] stringValue]];
    [_resolvedUrlRunnableMap setValue:startedTransactionResolverRunnable forKey:[[NSNumber numberWithInt:ADD_PAYMENT_METHOD_STARTED_TRANSACTION_3DS] stringValue]];
    
    [_resolvedUrlRunnableMap setValue:addPaymentMethodSuccessRunnable forKey:[[NSNumber numberWithInt:ADD_PAYMENT_METHOD_SUCCESS] stringValue]];
    
    [_resolvedUrlRunnableMap setValue:threeDSResolverRunnable forKey:[[NSNumber numberWithInt:ADD_PAYMENT_METHOD_THREE_DS] stringValue]];
    
    [_resolvedUrlRunnableMap setValue:timeoutResolverRunnable forKey:[[NSNumber numberWithInt:TIMEOUT] stringValue]];
    
    [_resolvedUrlRunnableMap setValue:transactionFailureResolverRunnable forKey:[[NSNumber numberWithInt:ADD_PAYMENT_METHOD_FAILURE] stringValue]];
}

-(int) resolve:(NSString *)request{
    for(id key in _resolvedUrlResultMap) {
        if([request rangeOfString:key].location != NSNotFound) {
            NSNumber* value = [_resolvedUrlResultMap objectForKey:key];
            return [value intValue];
        }
    }
    return UNKNOWN;
}
    
-(void) logTransactionInfo:(NSString *)loggingInfo {
    
    if (_delegate && [_delegate respondsToSelector:@selector(logTransactionInfo:)]) {
        [_delegate logTransactionInfo:loggingInfo];
    }
}

-(void) notifyDelegate:(int)resolvedCode request:(NSString *)request {
    [self logTransactionInfo:[NSString stringWithFormat:@"notifyDelegate: resolvedCode: %d", resolvedCode]];
    
    NSArray *testArray = request == nil ? nil: [request componentsSeparatedByString:@"/"];
    NSString *inputParam = (testArray == nil || testArray.count == 0)? nil: testArray[testArray.count - 1];
    
    NSString* key = [[NSNumber numberWithInt:resolvedCode] stringValue];
    
    id value = [_resolvedUrlRunnableMap objectForKey:key];
    
    if (value != nil) {
        id<RedirectResolverRunnable> runnable = (id<RedirectResolverRunnable>)value;
        [runnable run:resolvedCode request:request redirectResolver:self inputParam:inputParam];
    } else {
        [self logTransactionInfo:[NSString stringWithFormat:@"Unable to resolve %d", resolvedCode]];
    }
}

-(void) stopTimeoutTracking{
    if (_timer) {
        [self logTransactionInfo:@"Stoping timeout tracking"];
        [_timer invalidate];
        _timer = nil;
    }
}
-(void) startTimeoutTracking{
    
    [self logTransactionInfo:@"Starting timeout tracking"];
    
    _timer = [NSTimer scheduledTimerWithTimeInterval:1.0
                                              target:self
                                            selector:@selector(onTick:)
                                            userInfo:nil
                                             repeats:YES];
}

-(void)onTick:(NSTimer *)timer {
    _timePassed += _timeoutHandlerPeriod;
    if (_timePassed >= _timeout) {
        _timePassed = 0;
        [self logTransactionInfo:[NSString stringWithFormat:@"Transaction timeout with timeout: %ld", _timeout]];
        [self notifyDelegate:TIMEOUT request:nil];
    }
}

-(void) dealloc{
    [_timer invalidate];
    _timer = nil;
}
@end
