//
//  TransactionFailureResolverRunnable.m
//  WebtehPaymentLib
//
//  Created by Jasmin Suljic on 04/10/2018.
//

#import <Foundation/Foundation.h>
#import "TransactionFailureResolverRunnable.h"
#import "RedirectResolver.h"

@interface TransactionFailureResolverRunnable ()
@property(nonatomic, weak) id<SharedLifecycleDelegate> delegate;
@end

@implementation TransactionFailureResolverRunnable
- (id)init:(id<SharedLifecycleDelegate>)delegate {
    _delegate = delegate;
    return [self init];
}
- (void)run:(int)resolvedCode request:(NSString *)request redirectResolver:(RedirectResolver *)redirectResolver inputParam:(NSString *)inputParam {
    // If transaction is not in progress there is no need for transaction success handling
    // This is guard from multiple delegate calls when timeout is invoked and webview is not stopped.
    // After a while transaction success or transaction failure occurs.
    if(!redirectResolver.transactionInProgress){
        [redirectResolver logTransactionInfo:[NSString stringWithFormat:@"Transaction success invoked, but transactionInProgress = false. Ignoring."]];
        return;
    }
    
    redirectResolver.transactionInProgress = false;
    [redirectResolver stopTimeoutTracking];
    [_delegate hideProgress];
    NSDictionary* decodedInput = [InputResolver decode:inputParam];
    BusinessError* businessError = [BusinessError fromJson:decodedInput];
    
    if (businessError) {
        [redirectResolver logTransactionInfo:[NSString stringWithFormat:@"Transaction failure %@", businessError]];
        [self transactionFailure:businessError];
    } else{
        ApiError *apiError = [ApiError fromJson:decodedInput];
        [redirectResolver logTransactionInfo:[NSString stringWithFormat:@"Transaction failure %@", apiError]];
        [self transactionFailure:apiError];
    }
}

-(void) transactionFailure: (ApiError*) error {
    if (error){
        if (_delegate) {
            [_delegate handleTransactionError:error];
        }
    } else{
        if (_delegate) {
            [_delegate handleTransactionError:[ApiErrors libraryParsingError:@"Unable to parse api error"]];
        }
    }
}

@end
