//
//  WebtehWebView.m
//  pay
//
//  Created by Jasmin Suljic on 21/12/16.
//  Copyright © 2019 Monri Payments d.o.o. All rights reserved.
//

#import "WebtehWebView.h"
#import "PgwApi.h"
#import "RedirectResolver.h"
#import "WebtehWebViewDelegate.h"
#import "AddPaymentMethodLifecycleAdapter.h"
#import "AddPaymentMethodModel.h"
#import <WebKit/WebKit.h>

@interface WebtehWebView ()
    @property(nonatomic, strong) RedirectResolver *redirectResolver;
    @property(nonatomic, strong) WebtehWebViewDelegate *webtehWebViewDelegate;
@property(nonatomic, weak) id<SharedLifecycleDelegate> sharedDelegate;
    @property bool isDebug;
    @property long timeout;
@property bool initialized;


@end

@implementation WebtehWebView

-(void) logTransactionInfo:(NSString *)loggingInfo {
    if (_sharedDelegate && [_sharedDelegate respondsToSelector:@selector(logTransactionInfo:)]) {
        [_sharedDelegate logTransactionInfo:loggingInfo];
    }
}

- (instancetype)initWithCoder:(NSCoder *)coder
{
    // An initial frame for initialization must be set, but it will be overridden
    // below by the autolayout constraints set in interface builder.
    CGRect frame = [[UIScreen mainScreen] bounds];
    WKWebViewConfiguration *myConfiguration = [WKWebViewConfiguration new];
    
    // Set any configuration parameters here, e.g.
    // myConfiguration.dataDetectorTypes = WKDataDetectorTypeAll;
    
    self = [super initWithFrame:frame configuration:myConfiguration];
    
    // Apply constraints from interface builder.
    self.translatesAutoresizingMaskIntoConstraints = NO;
    
    return self;
}


-(void)initialize:(TransactionLifecycleAdapter *) transactionLifecycle
          isDebug:(bool) isDebug
          timeout:(long) timeout{
    
    self.isDebug = isDebug;
    self.redirectResolver = [[RedirectResolver alloc] init:transactionLifecycle timeout:timeout];
    self.initialized = true;
    self.webtehWebViewDelegate = [[WebtehWebViewDelegate alloc] init:self.redirectResolver];
    self.sharedDelegate = (id<SharedLifecycleDelegate>) transactionLifecycle.delegate;
    self.navigationDelegate = _webtehWebViewDelegate;
}

-(void) setupRedirectResolver: (AddPaymentMethodLifecycleAdapter *) adapter  {
    self.redirectResolver = [[RedirectResolver alloc] initAddPayment: adapter timeout:_timeout];
    self.sharedDelegate = (id<SharedLifecycleDelegate>) adapter.delegate;
    self.webtehWebViewDelegate = [[WebtehWebViewDelegate alloc] init:self.redirectResolver];
    self.navigationDelegate = _webtehWebViewDelegate;
}

-(void) requireInitialized {
    if (!_initialized) {
        [NSException raise:@"WebTehWebViewNotInitialized" format:@"WebTehWebView not initialized, initialize method not called!"];
    }
    
    if (_redirectResolver == nil) {
        [NSException raise:@"WebTehWebViewNotInitialized" format:@"RedirectResolver == null, initialize method not called!"];
    }
    
    if (_sharedDelegate == nil) {
        [NSException raise:@"WebTehWebViewNotInitialized" format:@"delegate == null, initialize method not called!"];
    }
}

- (void)initialize:(bool)isDebug timeout:(long)timeout {
    _isDebug = isDebug;
    _timeout = timeout;
    _initialized = true;
}

-(void) addCard: (AddCardModel *) addCardModel{
    [self logTransactionInfo:[NSString stringWithFormat:@"Add card loaded with order number: %@", addCardModel.orderNumber]];
    [self requireInitialized];
    [self loadRequest:[PgwApi addCard:[addCardModel toInput]]];
}
-(void) payCvv: (PayCvvModel *) payCvvModel{
    [self logTransactionInfo:[NSString stringWithFormat:@"Pay cvv loaded with order number: %@", payCvvModel.orderNumber]];
    [self requireInitialized];
    [self loadRequest:[PgwApi payCvv:[payCvvModel toInput]]];
}

-(void) addPaymentMethod:(AddPaymentMethodLifecycleAdapter *)adapter model:(AddPaymentMethodModel *)model {
    [self logTransactionInfo:[NSString stringWithFormat:@"AddPaymentMethod loaded with order number: %@", model.orderNumber]];
    [self setupRedirectResolver:adapter];
    [self requireInitialized];
    [self loadRequest:[PgwApi addPaymentMethod:[model toInput]]];
}



@end
