//
//  WebtehWebViewDelegate.m
//  pay
//
//  Created by Jasmin Suljic on 21/12/16.
//  Copyright © 2016 Webteh.hr. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "WebtehWebViewDelegate.h"
#import <WebKit/WebKit.h>

@interface WebtehWebViewDelegate()
    @property(strong,nonatomic) RedirectResolver *redirectResolver;
    @end

@implementation WebtehWebViewDelegate
    
-(id) init: (RedirectResolver *) redirectResolver{
    self.redirectResolver = redirectResolver;
    return [self init];
}
    
- (BOOL)webView:(WKWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType{
    [self notifyDelegate:request];
    return YES;
}
    
-(void) logTransactionInfo:(NSString *)loggingInfo {
        if(self.redirectResolver.delegate && [self.redirectResolver.delegate respondsToSelector:@selector(logTransactionInfo:)]){
            [self.redirectResolver.delegate logTransactionInfo:loggingInfo];
        }
}

-(void) notifyDelegate:(NSURLRequest*) request {
    [self.redirectResolver notifyDelegate:[self.redirectResolver resolve:request.URL.absoluteString] request:request.URL.absoluteString];
    
}

- (void)webView:(WKWebView *)webView decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler {
    [self notifyDelegate:navigationAction.request];
    decisionHandler(WKNavigationActionPolicyAllow);
}

-(void) didStartLoad:(WKWebView *)webView {
    if(self.redirectResolver.delegate && [self.redirectResolver.delegate respondsToSelector:@selector(webViewDidStartLoad:)]){
        
        [self logTransactionInfo:@"self.redirectResolver.delegate#webViewDidStartLoad"];
        [self.redirectResolver.delegate webViewDidStartLoad:webView];
    }
}

-(void) didFinishLoad:(WKWebView *)webView {
    if(self.redirectResolver.delegate && [self.redirectResolver.delegate respondsToSelector:@selector(webViewDidFinishLoad:)]){
        [self logTransactionInfo:@"self.redirectResolver.delegate#webViewDidFinishLoad"];
        [self.redirectResolver.delegate webViewDidFinishLoad:webView];
    }
}
    
-(void) webViewDidStartLoad:(WKWebView *)webView {
    [self didStartLoad:webView];
}

- (void)webView:(WKWebView *)webView didStartProvisionalNavigation:(WKNavigation *)navigation {
    [self didStartLoad:webView];
}
    
-(void) webViewDidFinishLoad:(WKWebView *)webView {
    [self didFinishLoad:webView];
}

- (void)webView:(WKWebView *)webView didFinishNavigation:(WKNavigation *)navigation {
    [self didFinishLoad:webView];
}
    
    
@end
